/* %W% %E% */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

// A layout manager that lays out components in either a vertical
// or horizontal arrangement giving each component a percentage
// of the arrangement.  I did this quickly at the end and it can
// certainly be improved.  For one, it doesn't check to ensure that
// the percentages total 100!  It's still pretty useful.

//----------------------------------------------------------------------------
// Change History:
//
// 12/98 created  rtw
//----------------------------------------------------------------------------


package ibmspace.client;

import java.lang.*;
import java.awt.*;
import java.util.*;


public class PercentLayout implements LayoutManager2
{
    static int VERT = 0;
    static int HORZ = 1;

    private int fAlign;

    private Vector fComponents = null;

    public PercentLayout (int align)
    {
	fAlign = align;
	fComponents = new Vector ();
    }

    public void addLayoutComponent (Component c, Object constraints)
    {
	fComponents.addElement (new ComponentInfo(c,(Float)constraints));
    }

    public void addLayoutComponent(String name, Component c)
    {
	// Not supported
    }
  
    public void removeLayoutComponent (Component c)
    {
	for (int i=0; i<fComponents.size(); i++) {
	    ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
	    if ( ci.fComponent == c) {
		fComponents.removeElement (ci);
	    }
	}
    }

    public float getLayoutAlignmentX (Container target)
    {
	return target.getAlignmentX ();
    }

    public float getLayoutAlignmentY (Container target)
    {
	return target.getAlignmentY ();
    }

    public void invalidateLayout (Container target)
    {
    }

    public Dimension preferredLayoutSize(Container target)
    {
	Dimension size = new Dimension (0,0);

	if ( fAlign == VERT ) {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getPreferredSize ();
		size.setSize (Math.max(size.width,cSize.width), size.height+cSize.height);
	    }
	} else {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getPreferredSize ();
		size.setSize (size.width+cSize.width, Math.max(size.height,cSize.height));
	    }
	}

	return size;
    }

    public Dimension minimumLayoutSize(Container target)
    {
	Dimension size = new Dimension (0,0);

	if ( fAlign == VERT ) {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getMinimumSize ();
		size.setSize (Math.max(size.width,cSize.width), size.height+cSize.height);
	    }
	} else {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getMinimumSize ();
		size.setSize (size.width+cSize.width, Math.max(size.height,cSize.height));
	    }
	}

	return size;
    }

    public Dimension maximumLayoutSize (Container target)
    {
	Dimension size = new Dimension (0,0);

	if ( fAlign == VERT ) {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getMaximumSize ();
		size.setSize (Math.max(size.width,cSize.width), size.height+cSize.height);
	    }
	} else {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		Dimension cSize = c.getMaximumSize ();
		size.setSize (size.width+cSize.width, Math.max(size.height,cSize.height));
	    }
	}

	return size;
    }
  
    public void layoutContainer (Container target)
    {
	Insets insets = target.getInsets();
	int x = insets.left;
	int y = insets.top;
	int width = target.getSize().width - (insets.left + insets.right);
	int height = target.getSize().height - (insets.top + insets.bottom);

	if ( fAlign == VERT ) {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		float p = ci.fPercentage;
		int ch = (int)(height*p);
		c.setLocation (x, y);
		c.setSize (width, ch);
		y += ch;
	    }
	} else {
	    for (int i=0; i<fComponents.size(); i++) {
		ComponentInfo ci = (ComponentInfo)fComponents.elementAt (i);
		Component c = ci.fComponent;
		float p = ci.fPercentage;
		int cw = (int)(width*p);
		c.setLocation(x, y);
		c.setSize (cw, height);
		x += cw;
	    }
	}

    }

}


class ComponentInfo
{
    public Component  fComponent = null;
    public float      fPercentage;

    public ComponentInfo (Component c, Float p)
    {
	fComponent = c;
	fPercentage = p.floatValue ();
    }
}
