/* %W% %E% */
/*
 * Licensed Materials - Property of IBM
 * RMI-IIOP v1.0
 * Copyright IBM Corp. 1998 1999  All Rights Reserved
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

//----------------------------------------------------------------------------
// Change History:
//
// 1/29/99  rtw   created
//----------------------------------------------------------------------------


package ibmspace.server;

import java.awt.Point;
import ibmspace.common.ID;
import ibmspace.common.PlanetView;



public class PlanetViewImpl implements PlanetView, java.io.Serializable
{
    private Player        fPlayer;
    private PlanetImpl    fPlanet;


    public PlanetViewImpl (Player player, PlanetImpl planet)
    {
	fPlayer = player;
	fPlanet = planet;
    }

    //
    // Planet Interface Methods
    //

    public ID getID ()
    {
	return fPlanet.getID ();
    }

    public String getName ()
    {
	return fPlanet.getName ();
    }

    public Point getCoordinates ()
    {
	return fPlanet.getCoordinates ();
    }

    public boolean hasSatelites ()
    {
	return fPlanet.hasSatelites ();
    }


    //
    // PlanetView Methods
    //

    public boolean isOwned ()
    {
	return ( fPlayer == fPlanet.getOwner() );
    }

    public long getMetal ()
    {
	return fPlanet.getMetal ();
    }

    public long getPopulation ()
    {
	if ( isOwned() ) {
	    Settlement settlement = fPlanet.getSettlement ();
	    if ( settlement != null ) {
		return settlement.getPopulation().size ();
	    } else {
		return 0;
	    }
	} else {
	    return -1;
	}
    }

    public long getIncome ()
    {
	if ( isOwned() ) {
	    Settlement settlement = fPlanet.getSettlement ();
	    if ( settlement != null ) {
		return settlement.getIncome ();
	    } else {
		return 0;
	    }
	} else {
	    return -1;
	}
    }

    public double getTemp ()
    {
	if ( isOwned() ) {
	    return fPlayer.getRelativeTempFor (fPlanet.getTemp());
	} else {
	    return -1;
	}
    }

    public double getGravity ()
    {
	if ( isOwned() ) {
	    return fPlayer.getRelativeGravityFor (fPlanet.getGravity());
	} else {
	    return -1;
	}
    }

    public double getSuitability ()
    {
	if ( isOwned() ) {
	    return fPlanet.getSuitabilityFor (fPlayer);
	} else {
	    return -1;
	}
    }

}

